function matlab_MSR_and_LEC(sig1,sig2,sig3,LUTfile)

% /!\ Dynamic of all images here (exept from le log() ones) are in [0 255];

L           = 3;                                        % Canonical value
                                                        % => gain limitatn
img_colr    = double(imread('input_0_sel.png'));
img_gray    = sum(img_colr,3)/3;

LUTfile_id  = fopen(LUTfile,'r');                       % Read DxO LUT
doplut      = NaN(32768,1);
for n=1:32768
    doplut(n)   = str2double(fgetl(LUTfile_id));
end
fclose(LUTfile_id);
doplut      = doplut(1:128:end)/128;                    % LUT Conversion:
LUT         = @(u)(doplut(round(u+1)));                 %      15 >> 8 bits

MSR_gray    = zeros(size(img_gray));
LEC_gmap    = zeros(size(img_gray));

sigma       = [str2double(sig1),str2double(sig2),str2double(sig3)];
N           = length(sigma);
[H,W]       = size(img_gray);
[Gh,Gw]     = meshgrid( -pi:(pi/W):+pi-(pi/W), -pi:(pi/H):+pi-(pi/H) );

for n=1:N
    % Fourrier Gaussian Filtering %                     %------------------                       
    gauss = exp(-(sigma(n)^2)*(Gh.^2 + Gw.^2)/2);       % FT{gaussian_sig}
    img_p = cat(1,img_gray,img_gray(end:-1:1,:));       % img 2x bigger
    img_p = cat(2,img_p,img_p(:,end:-1:1));             % img 4x bigger
    img_t = fftshift(fft2(img_p));                      % FT{img}
    img_f = abs(ifft2(fftshift(img_t.*gauss)));         % gauss. filtering
    img_f = img_f(1:H,1:W);                             % crop >> img_filt
    
    % Multiscale Retinex %
    MSR_gray = MSR_gray + (log(255*img_gray+1)-log(255*img_f+1))/N;
    
    % Local Exposure Correction %
    LEC_gmap = LEC_gmap + (LUT(img_f)./img_f)/N;
end
clear gauss img_p img_t img_f Gh Gw;

gain_max        = 255./max(img_colr,[],3);

% Multiscale Retinex %
MSR_gray        = ipolSCB(MSR_gray,1,1,255);            % Simp.Col.Balance
MSR_gainmap     = min(MSR_gray./img_gray,gain_max);     % To avoid col. sat
MSR_gainmap(MSR_gainmap>L) = L;                         % Limitation at *L*

MSR_colr(:,:,1) = img_colr(:,:,1).*MSR_gainmap;
MSR_colr(:,:,2) = img_colr(:,:,2).*MSR_gainmap;
MSR_colr(:,:,3) = img_colr(:,:,3).*MSR_gainmap;

% Local Exposure Correction %
LEC_gainmap     = min(LEC_gmap,gain_max);               % To avoid col. sat
LEC_gainmap(LEC_gainmap>L) = L;                         % Limitation at *L*

LEC_colr(:,:,1) = img_colr(:,:,1).*LEC_gainmap;
LEC_colr(:,:,2) = img_colr(:,:,2).*LEC_gainmap;
LEC_colr(:,:,3) = img_colr(:,:,3).*LEC_gainmap;

% Saving outputs %
imwrite(uint8(MSR_colr),'MSR_output_color.png');
imwrite(uint8(LEC_colr),'LEC_output_color.png');
imwrite(uint8(ipolSCB(MSR_gainmap,0,0,255)),'MSR_gainmap_min0max3.png');
imwrite(uint8(ipolSCB(LEC_gainmap,0,0,255)),'LEC_gainmap_min0max3.png');

end